from argparse import ArgumentParser

from twisted.internet import reactor

from .protocol import create_user, update_user, delete_user, fetch_user


def action_done(result):
    print(result)


def action_fail(error):
    print(error)


def create_action(args):
    data = {
        'fname': args.fname,
        'lname': args.lname,
    }

    d = create_user(args.host, args.port, data)

    d.addCallbacks(action_done, action_fail)


def update_action(args):
    data = {'id': args.id}

    if args.fname:
        data['fname'] = args.fname

    if args.lname:
        data['lname'] = args.lname

    d = update_user(args.host, args.port, data)

    d.addCallbacks(action_done, action_fail)


def delete_action(args):
    d = delete_user(args.host, args.port, args.id)

    d.addCallbacks(action_done, action_fail)


def fetch_action(args):
    d = fetch_user(args.host, args.port, args.id)

    d.addCallbacks(action_done, action_fail)


def main():
    parser = ArgumentParser()

    parser.add_argument('--host', type=str, default='127.0.0.1',
                        help='the remote host to connect to '
                             '(default is 127.0.0.1)')

    parser.add_argument('--port', type=int, default=8080,
                        help='the remote port to connect to '
                             '(default is 8080)')

    subs = parser.add_subparsers(dest='command')

    # Sub-commands should always be present.
    subs.required = True

    ###################################
    # "CREATE" command.
    ###################################

    create = subs.add_parser('create', help='create a new user')

    create.add_argument('--fname', required=True, help='first name')
    create.add_argument('--lname', required=True, help='last name')

    # Set command handler.
    create.set_defaults(action=create_action)

    ###################################
    # "UPDATE" command.
    ###################################

    update = subs.add_parser('update', help='update an existing user')

    update.add_argument('id', type=int, help='the user ID to update')

    update.add_argument('--fname', help='first name')
    update.add_argument('--lname', help='last name')

    # Set command handler.
    update.set_defaults(action=update_action)

    ###################################
    # "DELETE" command.
    ###################################

    delete = subs.add_parser('delete', help='delete an existing user')

    delete.add_argument('id', type=int, help='the user ID to delete')

    # Set command handler.
    delete.set_defaults(action=delete_action)

    ###################################
    # "FETCH" command.
    ###################################

    fetch = subs.add_parser('fetch', help='fetch an existing user')

    fetch.add_argument('id', type=int, nargs='?', default=None,
                       help='the user ID to fetch')

    # Set command handler.
    fetch.set_defaults(action=fetch_action)

    # Get all arguments.
    args = parser.parse_args()

    # Execute command handler.
    args.action(args)

    # Start the engine.
    reactor.run()


if __name__ == '__main__':
    main()
